@@ -41,7 +41,7 @@ $cmd = "cd qgvdial-$qver & perl build-files/version.pl __QTDIR__ $cmd";
 system($cmd);
 
 # Fix the QML files
-$cmd = "cd $basedir ; perl ./build-files/fixqml.pl ./qml";
+$cmd = "cd qgvdial-$qver & perl ./build-files/fixqml.pl ./qml";
 print "$cmd\n";
 system($cmd);
 
@@ -49,6 +49,7 @@ system($cmd);
 $cmd = "cd qgvdial-$qver & qmake & make release";
 system($cmd);
 
+# this is required for the old
 $cmd = "copy qgvdial-$qver\\src\\release\\qgvdial.exe I:\\Uv\\releases\\qgvdial\\win-install\\qgvdial\\bin";
 system($cmd);
 
