@@ -410,6 +410,7 @@ MainWindow::initQML ()
     QDeclarativeContext *ctx = this->rootContext();
     ctx->setContextProperty ("g_MainWidth", rect.width ());
     ctx->setContextProperty ("g_MainHeight", rect.height ());
+    ctx->setContextProperty ("g_bShowMsg", false);
 
     // Initialize the QML view
     this->setSource (QUrl ("qrc:/Main.qml"));
@@ -459,6 +460,8 @@ MainWindow::initQML ()
                                           const QString &)),
         this, SLOT   (onSigMosquittoChanges(bool, const QString &, int,
                                             const QString &)));
+    QObject::connect (gObj, SIGNAL (sigMsgBoxDone(bool)),
+                      this, SLOT (onSigMsgBoxDone(bool)));
 
 #if DESKTOP_OS
     this->setFixedSize (this->size ());
@@ -479,6 +482,7 @@ MainWindow::initQMLGlobals ()
 void
 MainWindow::doLogin ()
 {
+    OsDependent &osd = Singletons::getRef().getOSD ();
     GVAccess &webPage = Singletons::getRef().getGVAccess ();
     QVariantList l;
 
@@ -490,17 +494,18 @@ MainWindow::doLogin ()
         l += strPass;
 
         setStatus ("Logging in...", 0);
+
+        osd.setLongWork (this, true);
+
         // webPage.workCompleted -> this.loginCompleted
         if (!webPage.enqueueWork (GVAW_login, l, this,
                 SLOT (loginCompleted (bool, const QVariantList &))))
         {
             qWarning ("Login returned immediately with failure!");
+            osd.setLongWork (this, false);
             break;
         }
 
-        OsDependent &osd = Singletons::getRef().getOSD ();
-        osd.setLongWork (this, true);
-
         bOk = true;
     } while (0); // End cleanup block (not a loop)
 
@@ -561,25 +566,14 @@ MainWindow::loginCompleted (bool bOk, const QVariantList &varList)
 
     if (!bOk)
     {
-        setStatus ("User login failed");
-
-        // Cleanup if any
-        QMessageBox *msgBox = new QMessageBox(QMessageBox::Critical,
-                           "Invalid username or password",
-                           "Username or password not recognized",
-                           QMessageBox::Close,
-                           this);
-        msgBox->setModal (false);
-        QObject::connect (
-            msgBox, SIGNAL (buttonClicked (QAbstractButton *)),
-            this  , SLOT   (msgBox_buttonClicked (QAbstractButton *)));
-        msgBox->show ();
-
         QVariantList l;
         logoutCompleted (true, l);
 
         OsDependent &osd = Singletons::getRef().getOSD ();
         osd.setLongWork (this, false);
+
+        setStatus ("User login failed", 30*1000);
+        this->showMsgBox ("User login failed");
     }
     else
     {
@@ -684,15 +678,6 @@ MainWindow::systray_activated (QSystemTrayIcon::ActivationReason reason)
     }
 }//MainWindow::systray_activated
 
-void
-MainWindow::msgBox_buttonClicked (QAbstractButton *button)
-{
-    if (NULL != button->parent ())
-    {
-        button->parent()->deleteLater ();
-    }
-}//MainWindow::msgBox_buttonClicked
-
 void
 MainWindow::on_actionE_xit_triggered ()
 {
@@ -717,15 +702,7 @@ MainWindow::getContactsDone (bool bOk)
 {
     if (!bOk)
     {
-        QMessageBox *msgBox = new QMessageBox(QMessageBox::Critical,
-                           "Error",
-                           "Contacts retrieval failed",
-                           QMessageBox::Close);
-        msgBox->setModal (false);
-        QObject::connect (
-            msgBox, SIGNAL (buttonClicked (QAbstractButton *)),
-            this  , SLOT   (msgBox_buttonClicked (QAbstractButton *)));
-        msgBox->show ();
+        this->showMsgBox ("Contacts retrieval failed");
         setStatus ("Contacts retrieval failed");
     }
 }//MainWindow::getContactsDone
@@ -794,17 +771,7 @@ MainWindow::getInfoFrom (const QString &strNumber,
 
         if (!dbMain.getContactFromLink (info))
         {
-            QMessageBox *msgBox = new QMessageBox(QMessageBox::Critical,
-                                        "Get Contact failure",
-                                        "Failed to get contact information",
-                                        QMessageBox::Close,
-                                        this);
-            msgBox->setModal (false);
-            QObject::connect (
-                msgBox, SIGNAL (buttonClicked (QAbstractButton *)),
-                this  , SLOT   (msgBox_buttonClicked (QAbstractButton *)));
-            msgBox->show ();
-
+            this->showMsgBox ("Failed to get contact information");
             return (false);
         }
 
@@ -1204,17 +1171,8 @@ MainWindow::dialComplete (bool bOk, const QVariantList &params)
         }
         else
         {
-            setStatus ("Dialing failed", 3);
-            QMessageBox *msgBox = new QMessageBox(QMessageBox::Critical,
-                               "Dial failure",
-                               "Dialing failed",
-                               QMessageBox::Close,
-                               this);
-            msgBox->setModal (false);
-            QObject::connect (
-                msgBox, SIGNAL (buttonClicked (QAbstractButton *)),
-                this  , SLOT   (msgBox_buttonClicked (QAbstractButton *)));
-            msgBox->show ();
+            setStatus ("Dialing failed", 10*1000);
+            this->showMsgBox ("Dialing failed");
         }
     }
     else
@@ -1311,17 +1269,8 @@ MainWindow::sendSMS (const QStringList &arrNumbers, const QString &strText)
 
     if (0 != arrFailed.size ())
     {
-        QMessageBox *msgBox = new QMessageBox(QMessageBox::Critical,
-                                    "SMS could not be sent to the following:",
-                                    arrFailed.join (", "),
-                                    QMessageBox::Close,
-                                    this);
-        msgBox->setModal (false);
-        QObject::connect (
-            msgBox, SIGNAL (buttonClicked (QAbstractButton *)),
-            this  , SLOT   (msgBox_buttonClicked (QAbstractButton *)));
-        msgBox->show ();
-
+        this->showMsgBox (QString("Failed to send %1 SMS")
+                                 .arg (arrFailed.size ()));
         msg = QString("Could not send a text to %1")
                 .arg (arrFailed.join (", "));
         setStatus (msg);
@@ -1403,17 +1352,8 @@ MainWindow::gotAllRegisteredPhones (bool bOk, const QVariantList &)
     do { // Begin cleanup block (not a loop)
         if (!bOk)
         {
-            QMessageBox *msgBox = new QMessageBox(
-                    QMessageBox::Critical,
-                    "Error",
-                    "Failed to retrieve registered phones",
-                    QMessageBox::Close);
-            msgBox->setModal (false);
-            QObject::connect (
-                    msgBox, SIGNAL (buttonClicked (QAbstractButton *)),
-                    this  , SLOT   (msgBox_buttonClicked (QAbstractButton *)));
-            msgBox->show ();
-            setStatus ("Failed to retrieve all registered phones");
+            this->showMsgBox ("Failed to retrieve registered phones");
+            setStatus ("Failed to retrieve registered phones");
             break;
         }
 
@@ -1745,3 +1685,18 @@ MainWindow::onMqThreadFinished ()
     }
 #endif
 }//MainWindow::onMqThreadFinished
+
+void
+MainWindow::showMsgBox (const QString &strMessage)
+{
+    QDeclarativeContext *ctx = this->rootContext();
+    ctx->setContextProperty ("g_bShowMsg", true);
+    ctx->setContextProperty ("g_strMsgText", strMessage);
+}//MainWindow::showMsgBox
+
+void
+MainWindow::onSigMsgBoxDone (bool /*ok*/)
+{
+    QDeclarativeContext *ctx = this->rootContext();
+    ctx->setContextProperty ("g_bShowMsg", false);
+}//MainWindow::onSigMsgBoxDone
